@@ -235,7 +235,7 @@ class WebServer(objects.Thread):
 	def start_thread(self, thread, group="daemons", id=None):
 		response = self.set(group=group, id=id, data=thread)
 		if not response.success: return response
-		respose = thread.start()
+		response = thread.start()
 		if response != None:
 			try: success = bool(response["success"])
 			except: success = True
