@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # defaults.
-import os,sys,json,pathlib,platform
+import os,sys,json,pathlib,platform,time
 def __get_argument__(argument, required=True, index=1, default=None):
 
 	# check presence.
