@@ -17,7 +17,7 @@ class Spawn(objects.Object):
 		# asynchronous.
 		async_=False,
 		# the log level.
-		log_level=0,
+		log_level=defaults.log_level,
 		# additional attributes.
 		attributes={},
 		# system options.
@@ -48,7 +48,7 @@ class Spawn(objects.Object):
 			return r3sponse.error(f"Failed to spawn {self.response_str}, error: {e}.")
 		# check exit status.
 		if self.child.exitstatus not in [0, None]:
-			return OutputObject(error=f"{self.response_str} returned exit status: [{spawn.child.exitstatus}].")
+			return OutputObject(error=f"{self.response_str} returned exit status: [{spawn.child.exitstatus}] (output: {self.read(wait=False, __safe__=True).output}).")
 		self.__output__ = ""
 		return r3sponse.success(f"Successfully spawned {self.response_str}.")
 		#
@@ -159,6 +159,11 @@ class Spawn(objects.Object):
 		wait=False, 
 		# the timeout, leave None for no timeout.
 		timeout=None,
+		# the live boolean (bool) (prints live logs to console when enabled) (leave None to use self.log_level >= 0).
+		live=None, 
+		# system variables.
+		#   safe True always a response.output variable upon error the response.output is "".
+		__safe__=False,
 	):
 
 		# clean line.
@@ -170,35 +175,67 @@ class Spawn(objects.Object):
 					if secret not in ["", " ", "  ", "   ", "    "] and len(line) >= len(strip_last) and line[-len(strip_last):] == strip_last: 
 						return line[:-len(strip_last)]
 			return line
+		def clean_output(output):
+			_output_, b = "", output.replace("\r","").replace("\t","").split("\n")
+			c, m = 0, len(b)-1
+			for line in b: 
+				l = clean_line(line)
+				if l not in [None]:
+					if c < m: _output_ += l+"\n"
+					else: _output_ += l
+				c += 1
+			return _output_
+		def log_output(output):
+			while True:
+				if len(output) >= 1 and output[len(output)-1] == "\n": output = output[:-1]
+				else: break	
+			print(output)
+
+		# checks.
+		if live == None: live = self.log_level >= 0
 
 		# old timeout.
 		old = self.child.timeout
 
 		# version 2: reads only the printed lines and stops when the timeout is hit so there is no timeout required. 
+		output_logs_allowed = True
 		if not wait:
 			self.child.timeout = 0.25
-			output = ""
-			for line in range(100000):
+			output_logs_allowed = False
+			output, c = "", 0
+			for line in range(1000000000):
 				try:
 					#if self.async_:
 					#	g = yield from self.child.readline(line, async_=True)
 					#	output += next(g).decode()
 					#else:
-					output += self.child.readline(line).decode()
+					new = self.child.readline(line).decode()
+					if new == "": 
+						c += 1
+						if c >= 100: break
+					else: 
+						output_logs_allowed = True
+						c = 0
+						output += new
 				except pexpect.exceptions.TIMEOUT:
 					break
 				except pexpect.exceptions.EOF:
 					break
 				except Exception as e:
 					if timeout != None: self.child.timeout = old
+					e = str(e)
 					if self.log_level <= 0: e = str(e).split("\n")[0] # strip the full child from the pexpect error message.
 					while True:
 						if len(e) >= 1 and e[len(e)-1] == ".": e = e[:-1]
 						else: break
 					if "Timeout" in str(e):
-						return r3sponse.error(f"{e} (most likely the command is still expecting input).")
+						response = r3sponse.error(f"{e} (most likely the command is still expecting input).")
+						if __safe__: response.output = ""
+						return response
 					else:
-						return r3sponse.error(f"{e}.")	
+						response = r3sponse.error(f"{e}.")	
+						if __safe__: response.output = ""
+						return response
 
 		# version 1: throws an error when unable to read output if timeout is defined, so it can be used to detect if input is expected. 
 		else:
@@ -213,34 +250,36 @@ class Spawn(objects.Object):
 				#	lines = next(x)
 				#else:
 				lines = self.child.readlines()
-				for i in lines: output += i.decode()
+				if lines == []:
+					output_logs_allowed = False
+				else:
+					for i in lines: output += i.decode()
 			except Exception as e:
 				self.child.timeout = old
+				e = str(e)
 				if self.log_level <= 0: e = str(e).split("\n")[0] # strip the full child from the pexpect error message.
 				while True:
 					if len(e) >= 1 and e[len(e)-1] == ".": e = e[:-1]
 					else: break
 				if "Timeout" in str(e):
-					return r3sponse.error(f"{e} (most likely the command is still expecting input).")
+					response = r3sponse.error(f"{e} (most likely the command is still expecting input).")
+					if __safe__: response.output = ""
+					return response
 				else:
-					return r3sponse.error(f"{e}.")
+					response = r3sponse.error(f"{e}.")
+					if __safe__: response.output = ""
+					return response
 
 		# clean output.
-		_output_, b = "", output.replace("\r","").replace("\t","").split("\n")
-		c, m = 0, len(b)-1
-		for line in b: 
-			l = clean_line(line)
-			if l not in [None]:
-				if c < m: _output_ += l+"\n"
-				else: _output_ += l
-			c += 1
-		output = _output_
+		output = clean_output(output)
+		if output_logs_allowed and live: log_output(output)
 
 		# handler.
 		self.child.timeout = old
 		self.__output__ += output
 		return r3sponse.success("Successfully retrieved the child's output.", {
 			"output":str(self.__output__),
+			"new_output":output,
 		})
 
 
@@ -269,39 +308,36 @@ class Spawn(objects.Object):
 		#
 
 	# wait for the process to finish.
-	def wait(self, timeout=None):
-			
-		# version 2.
-		# more stable.
-		response = self.read(wait=True, timeout=timeout)
-		if response.success:
-			while True:
-				if len(response.output) > 0 and response.output[len(response.output)-1] == "\n": response.output = response.output[:-1]
-				else: break
-		return response
+	def wait(self, 
+		# the live boolean (bool) (prints live logs to console when enabled) (leave None to use self.log_level >= 0).
+		live=None, 
+		sleeptime=3,
+		# the timeout (leave None to ignore).
+		timeout=None,
+	):
+		
+		# checks.
+		if live == None: live = self.log_level >= 0
 
-		"""
-		# version 1.
-		# does not work properly.
-		# handle output.
-		if timeout != None:
-			old = self.child.timeout
-			self.child.timeout = timeout
-		exit_status = None
-		try:
-			#if self.async_:
-			#	g = yield from self.child.wait(async_=True)
-			#	l = next(g)
-			#else:
-			exit_status = self.child.wait()
-		except Exception as e:
-			if timeout != None: self.child.timeout = old
-			return r3sponse.error(f"Failed to await process [{self.command}], error: {e}.")
-		if timeout != None: self.child.timeout = old
-		if exit_status not in [0, None]:
-			return r3sponse.error(f"{self.response_str} returned exit status: [{exit_status}].")
-		return r3sponse.success(f"Successfully awaited process [{self.command}].")
-		"""
+		# live
+		if live:
+			while self.running:
+				response = self.read(wait=False, live=live)
+				if not response.success: return response
+				if self.running:
+					time.sleep(sleeptime)
+			return self.wait(live=False, timeout=timeout)
+
+		# not live.
+		else:
+
+			# read & wait with no timeout
+			response = self.read(wait=True, timeout=timeout)
+			if response.success:
+				while True:
+					if len(response.output) > 0 and response.output[len(response.output)-1] == "\n": response.output = response.output[:-1]
+					else: break
+			return response
 
 		#
 
@@ -310,7 +346,7 @@ class Spawn(objects.Object):
 		response = self.read(wait=False)
 		if not response.success:  return response
 		if self.exit_status not in [0, None]:
-			return r3sponse.error(f"{self.response_str} returned exit status: [{self.exit_status}].")
+			return r3sponse.error(f"{self.response_str} returned exit status: [{self.exit_status}] (output: {self.read(wait=False, __safe__=True).output}).")
 		return r3sponse.success(f"{self.response_str} is not crashed.")
 
 	# is expecting input.
@@ -394,7 +430,7 @@ class OutputObject(ResponseObject):
 		# the error message (param #3).
 		error=None,
 		# the log level.
-		log_level=syst3m.defaults.options.log_level,
+		log_level=defaults.log_level(default=-1),
 	):
 		dictionary = {
 			"success":error == None,
@@ -411,7 +447,7 @@ class OutputObject(ResponseObject):
 			except AttributeError: self.output = ""
 		else: self.output = str(self.error) # keep this way for system functions.
 		self.lines = self.output.split("\n")
-		if self.log_level >= 0: print(self.output)
+		#if self.log_level >= 0: print(self.output)
 
 	# representations.
 	def __len__(self):
@@ -517,7 +553,7 @@ def execute(
 	#
 	# Logging.
 	#   the log level.
-	log_level=defaults.options.log_level,
+	log_level=defaults.log_level(default=-1),
 	#
 	# System functions.
 	#   add additional attributes to the spawn object.
@@ -658,7 +694,10 @@ def execute(
 									str_input = Array(list(_input_.keys())).string(joiner=", ")
 								else: 
 									str_input = f"[{_input_}]"
-								return OutputObject(error=f"Failed to send one of the expected input(s) {str_input}, error: {response.error}", log_level=log_level)
+								if optional:
+									break
+								else:
+									return OutputObject(error=f"Failed to send one of the expected input(s) {str_input}, error: {response.error}", log_level=log_level)
 
 				# send one of the keys (dict instance).
 				elif isinstance(input, (dict, Dictionary)):
@@ -666,15 +705,15 @@ def execute(
 					if not response.success: 
 						if "End of file" in response.error:
 							error_end_of_file = True
-						else: return OutputObject(error=f"Failed to send one of the expected input(s) {str_input}, error: {response.error}", log_level=log_level)
+						elif not optional:
+							return OutputObject(error=f"Failed to send one of the expected input(s) {str_input}, error: {response.error}", log_level=log_level)
 
 				""" check no input left (does not work properly).
 				if not error_end_of_file and spawn.expecting:
 					try: after = spawn.child.after.decode()
 					except : after = spawn.child.after
 					return OutputObject(error=f"Failed to execute {response_str}, still expecting: [{after}].", log_level=log_level)
-				"""
-			
+				"""		
 
 		# do not get or kill when async.
 		output = None
@@ -708,18 +747,18 @@ def execute(
 			if not response.success: 
 				if not response.success: return OutputObject(error=f"Failed to retrieve output from spawn {response_str}, error: {response.error}", log_level=log_level)
 			if spawn.child.exitstatus not in [0, None]:
-				return OutputObject(error=f"{response_str} returned exit status: [{spawn.child.exitstatus}].", log_level=log_level)
+				return OutputObject(error=f"{response_str} returned exit status: [{spawn.child.exitstatus}] (output: {self.read(wait=False, __safe__=True).output}).", log_level=log_level)
 
 			# await async.
 			if wait:
 
 				# await.
-				response = spawn.wait(wait=True)
+				response = spawn.wait()
 				if not response.success: return OutputObject(error=response.error, log_level=log_level) # exit status.
 				output = response.output
 
 				if spawn.child.exitstatus not in [0, None]:
-					return OutputObject(error=f"{response_str} returned exit status: [{spawn.child.exitstatus}].", log_level=log_level)
+					return OutputObject(error=f"{response_str} returned exit status: [{spawn.child.exitstatus}] (output: {self.read(wait=False, __safe__=True).output}).", log_level=log_level)
 				
 				# check kill.
 				if kill and spawn.running:
