@@ -13,27 +13,55 @@ vars = r3sponse.ResponseObject({
 	"user": os.environ.get("USER"),
 	"owner": os.environ.get("USER"),
 	"group": "root",
-	"media": f"/media/{os.environ.get('USER')}",
-	"homes": gfp.base(path=os.environ.get('HOME')),
-	"home": gfp.clean(os.environ.get('HOME')+"/"),
-	"python3":"python3",
-	"executable":sys.executable,
+	"executable":FilePath(sys.executable),
+	"media": Directory(f"/media/{os.environ.get('USER')}"),
+	"homes": Directory(gfp.base(path=os.environ.get('HOME'))),
+	"home": Directory(gfp.clean(os.environ.get('HOME')+"/")),
+	"pwd":Directory(gfp.clean(utils.__execute_script__("pwd").replace("\n",""))),
+	"site_packages":Directory(gfp.base(SOURCE_PATH)),
 })
 if vars.os in ["darwin"]:  
 	vars.os = "macos"
 	vars.group = "staff"
 	vars.media = f"/Volumes/"
 	vars.python3 = f"/usr/bin/python3"
+if vars.site_packages.fp.name() not in ["site-packages", "dist-packages"]: 
+	vars.site_packages = env.get("SITE_PACKAGES", default=None)
+	if vars.site_packages == None:
+		raise ValueError(f"<syst3m.defaults>: Define environment variable [SITE_PACKAGES] (syst3m developer mode) (syst3m base: {gfp.base(SOURCE_PATH)}).")
 
 # universal options.
 options = r3sponse.ResponseObject({
-	"interactive": not cl1.arguments_present(["--non-interactive"]) and env.get("INTERACTIVE", format=bool, default=False) == True,
+	"log_level": int(cl1.get_argument("--log-level", required=False, default=env.get_integer("LOG_LEVEL", default=0))),
 	"cli": env.get_boolean("CLI", default=False),
 	"json": cl1.arguments_present(["-j", "--json"]),
-	"log_level": int(cl1.get_argument("--log-level", required=False, default=env.get_integer("LOG_LEVEL", default=0))),
-	"forced": cl1.arguments_present(["--forced", "-f"], default=env.get("FORCED", default=False, format=bool)),
+	"interactive": not cl1.arguments_present(["--non-interactive"]) and env.get("INTERACTIVE", format=bool, default=False) == True,
+	"forced": cl1.arguments_present(["--forced", "-f", "--force"], default=env.get("FORCED", default=False, format=bool)),
+	"quiet": cl1.arguments_present(["--quiet", "-q", "-s", "--silent"], default=env.get("QUIET", default=False, format=bool)),
 })
 
+# logging.
+if options.log_level >= 1:
+	r3sponse.log(f"{ALIAS}:")
+	r3sponse.log(f"  * vars: ")
+	r3sponse.log(f"    - os: {vars.os}")
+	r3sponse.log(f"    - user: {vars.user}")
+	r3sponse.log(f"    - owner: {vars.owner}")
+	r3sponse.log(f"    - group: {vars.group}")
+	r3sponse.log(f"    - executable: {vars.executable}")
+	r3sponse.log(f"    - media: {vars.media}")
+	r3sponse.log(f"    - homes: {vars.homes}")
+	r3sponse.log(f"    - home: {vars.home}")
+	r3sponse.log(f"    - pwd: {vars.pwd}")
+	r3sponse.log(f"    - site_packages: {vars.site_packages}")
+	r3sponse.log(f"  * options: ")
+	r3sponse.log(f"    - log_level: {options.log_level}")
+	r3sponse.log(f"    - cli: {options.cli}")
+	r3sponse.log(f"    - json: {options.json}")
+	r3sponse.log(f"    - interactive: {options.interactive}")
+	r3sponse.log(f"    - forced: {options.forced}")
+	r3sponse.log(f"    - quiet: {options.quiet}")
+
 # get operating system.
 def operating_system(supported=["*"]):
 	os = vars.os
@@ -86,7 +114,7 @@ def source_path(path, back=1):
 
 # get log level.
 def log_level(default=0):
-	return int(cl1.get_argument("--log-level", required=False, default=env.get_integer("LOG_LEVEL", default=0)))
+	return int(cl1.get_argument("--log-level", required=False, default=env.get_integer("LOG_LEVEL", default=default)))
 
 # get current working directory.
 def pwd():
