@@ -69,7 +69,7 @@ class User(object):
 				return r3sponse.success(f"Successfully created user [{self.username}].")
 
 			else:
-				return r3sponse.error(String(str(output).replace("useradd: ", "").replace("\n", ". ")).capitalized_word())
+				return r3sponse.error(String(output.output.replace("useradd: ", "").replace("\n", ". ")).capitalized_word())
 
 		# handle macos.
 		elif OS in ["macos"]:
@@ -244,7 +244,7 @@ class Group(object):
 				return r3sponse.success(f"Successfully created group [{self.name}].")
 
 			else:
-				return r3sponse.error(String(str(output).replace("groupadd: ", "").replace("\n", ". ")).capitalized_word())
+				return r3sponse.error(String(output.output.replace("groupadd: ", "").replace("\n", ". ")).capitalized_word())
 
 		# handle macos.
 		elif OS in ["macos"]:
@@ -339,7 +339,7 @@ class Group(object):
 			for user in users:
 				output = console.execute(f"sudo deluser {user} {self.name}")
 				if output != "":
-					return r3sponse.error(String(str(output).replace("deluser: ", "").replace("\n", ". ")).capitalized_word())
+					return r3sponse.error(String(output.output.replace("deluser: ", "").replace("\n", ". ")).capitalized_word())
 
 		# handle macos.
 		elif OS in ["macos"]:
@@ -364,8 +364,8 @@ class Group(object):
 			for user in users:
 				output = console.execute(f"sudo usermod -a -G {self.name} {user}")
 				if not output.success: return output
-				if str(output) != "":
-					return r3sponse.error(String(str(output).replace("usermod: ", "").replace("\n", ". ")).capitalized_word())
+				if output.output != "":
+					return r3sponse.error(String(output.output.replace("usermod: ", "").replace("\n", ". ")).capitalized_word())
 
 		# handle macos.
 		elif OS in ["macos"]:
